﻿using System.Collections.ObjectModel;
using System.Drawing.Printing;
using System.Windows;
using System.Windows.Controls;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WpfDesktopEdition;

namespace CSHowDoISamples
{
    public partial class DisplayASimpleMap : UserControl
    {
        public DisplayASimpleMap()
        {
            InitializeComponent();
        }

        private void WpfMap_Loaded(object sender, RoutedEventArgs e)
        {
            Map1.MapUnit = GeographyUnit.DecimalDegree;
            WorldMapKitLayer lTest = new WorldMapKitLayer(WorldMapKitLayerType.WorldMapKitLayer);
            lTest.MapType = WorldMapKitMapType.AerialWithLabels;
            LayerOverlay overlayThinkGeo = new LayerOverlay();
            overlayThinkGeo.Name = "ThinkGeo";
            overlayThinkGeo.Layers.Add(lTest);
            Map1.Overlays.Add(overlayThinkGeo);

            //LayerOverlay overlayCache = new LayerOverlay();
            //overlayCache.Name = "TileCache";
            //overlayCache.TileType = TileType.MultipleTile;
            //overlayCache.TileCache = new FileBitmapTileCache(<cache path>, <cache id>);
            //Map1.Overlays.Add(overlayCache);

            PrinterInteractiveOverlay printerInteractiveOverlay = new PrinterInteractiveOverlay();
            Map1.InteractiveOverlays.Add("PrintPreviewOverlay", printerInteractiveOverlay);
            Map1.InteractiveOverlays.MoveToBottom("PrintPreviewOverlay");
            PagePrinterLayer pagePrinterLayer = new PagePrinterLayer(PrinterPageSize.AnsiA, PrinterOrientation.Portrait);
            pagePrinterLayer.Open();
            printerInteractiveOverlay.PrinterLayers.Add("PageLayer", pagePrinterLayer);
        }

        private void button_Click(object sender, RoutedEventArgs e)
        {
            RectangleShape rOrigExtent = Map1.CurrentExtent;
            GeographyUnit eOrigUnit = Map1.MapUnit;
            Map1.MapUnit = GeographyUnit.Meter;
            PrinterInteractiveOverlay printerInteractiveOverlay = (PrinterInteractiveOverlay)Map1.InteractiveOverlays["PrintPreviewOverlay"];
            PagePrinterLayer pagePrinterLayer = (PagePrinterLayer)printerInteractiveOverlay.PrinterLayers["PageLayer"];

            foreach (Overlay o in Map1.Overlays)
            {
                MapPrinterLayer mapPrinterLayer = new MapPrinterLayer();
                mapPrinterLayer.MapUnit = eOrigUnit;
                mapPrinterLayer.MapExtent = rOrigExtent;
                mapPrinterLayer.BackgroundMask = new AreaStyle(new GeoPen(GeoColor.StandardColors.Black, 1));
                mapPrinterLayer.Open();
                RectangleShape pageBoundingbox = pagePrinterLayer.GetPosition(PrintingUnit.Inch);
                mapPrinterLayer.SetPosition(7, 10, pageBoundingbox.GetCenterPoint().X, pageBoundingbox.GetCenterPoint().Y, PrintingUnit.Inch);
                foreach (Layer l in ((LayerOverlay)o).Layers)
                {
                    mapPrinterLayer.Layers.Add(l);
                }
                printerInteractiveOverlay.PrinterLayers.Add(mapPrinterLayer);
            }

            PrintDialog printDialog = new PrintDialog();
            PrintDocument printDocument = new PrintDocument();
            printDocument.DefaultPageSettings.Landscape = true;
            if (pagePrinterLayer.Orientation == PrinterOrientation.Portrait)
            {
                printDocument.DefaultPageSettings.Landscape = false;
            }

            PrinterGeoCanvas printerGeoCanvas = new PrinterGeoCanvas();
            printerGeoCanvas.DrawingArea = printDocument.DefaultPageSettings.Bounds;
            printerGeoCanvas.BeginDrawing(printDocument, pagePrinterLayer.GetBoundingBox(), Map1.MapUnit);

            // Loop through all of the PrintingLayer in the PrinterInteractiveOverlay and print all of the
            // except for the PagePrinterLayer                
            Collection<SimpleCandidate> labelsInAllLayers = new Collection<SimpleCandidate>();
            foreach (PrinterLayer printerLayer in printerInteractiveOverlay.PrinterLayers)
            {
                printerLayer.IsDrawing = true;
                if (!(printerLayer is PagePrinterLayer))
                {
                    printerLayer.Draw(printerGeoCanvas, labelsInAllLayers);
                }
                printerLayer.IsDrawing = false;
            }
            printerGeoCanvas.Flush();

            Map1.MapUnit = eOrigUnit;

            System.Windows.Forms.PrintPreviewDialog printPreviewDialog = new System.Windows.Forms.PrintPreviewDialog();
            printPreviewDialog.Document = printDocument;
            System.Windows.Forms.DialogResult dialogResult = printPreviewDialog.ShowDialog();
        }
    }
}
